/* vi: set sw=4 ts=4: */	
/***************************************************************************
 *        test_sram.c  -  description
 *        -------------------
 *	begin       : Mon Apr 29 2002
 *  license     : GPL 
 *  email       : f.callaghan@ieee.org
 *
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <sched.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <asm/page.h>
#include <getopt.h>

/********* THIS SHOULD BE IN THE HEADER FILE ***********/
#define SRAM_BASE 0xD0000	// Ram physical addr.
#define SRAM_SIZE 0x8000	// 32K

unsigned char *sram_open(void);
unsigned char *sram_base_ptr;

/*** Set Up Your Structures ***/
struct entry {
	char name[80];
	int age;
};
struct table {
	struct entry entry[100];
};
struct table* table_ptr; 

int sram_test(int read); // test routine only.
/******************************************************/

int main(int argc, char **argv) {
	int i,c;
	struct table;

   	sram_base_ptr = sram_open(); // Do all the magic!
	
	table_ptr =  (struct table*)sram_base_ptr;


	while((c = getopt(argc, argv, "r:wvt")) != -1) {
		switch(c) {
			case 't':
			case 'v':
				// Write test patten.
				if (sram_test(0)) {
				      printf("failed write test\n");
			    	  exit(-1);
				}
				// Read & Check patten.
				if (c == 't') {
					if (sram_test(1)) {
				    	printf("failed read test\n");
						exit(-1);
					}
				} else {
					if (sram_test(2)) {
				    	printf("failed read test\n");
						exit(-1);
					}
				}
				// All's good.
				printf("Good SRAM ;)\n");
				break;
	
				// Write test database.
			case 'w':
				printf("Writing Entry's\n");
				table_ptr->entry[1].age = 21;
				strcpy( table_ptr->entry[1].name, "Frank the Great");
				table_ptr->entry[2].age = 22;
				strcpy( table_ptr->entry[2].name, "Lou the Good");
				break;
	
				// Read a test entry.
			case 'r':
				c = *optarg - '0';
				if (c<1 || c>2) {
					printf("Only two records [1-2]\n");
					exit(1);
				}
				printf("Name: %s, Age: %d\n",  table_ptr->entry[c].name,  table_ptr->entry[c].age);
				break;
				
			default:
				printf("Usage: sram_test [CMD] [ARG]\n" 			
					"\t t\ttest sram (write & read back)\n" 	
					"\t w\ttest sram (write data records)\n" 
					"\t r [1-2]\ttest sram (read back data record 'n')\n\n"
				);
		}
	}
	return 0;
}

int sram_test(int read) {
	int i;
	unsigned char *ptr;
	int *iptr;

	if (!read) {
		printf("Writing test patterns to SRAM\n");
		memset((void*)sram_base_ptr, 0x00, SRAM_SIZE); // Clean All to Zero's
		for(i=0, ptr=sram_base_ptr; i<SRAM_SIZE; i++, ptr++) {
			if(*ptr != 0) return(-1);
		}
		memset((void*)sram_base_ptr, 0xff, SRAM_SIZE); // set All to 1's
		for(i=0, ptr=sram_base_ptr; i<SRAM_SIZE; i++, ptr++) {
			if(*ptr != 0xff) return(-1);
		}
		memset((void*)sram_base_ptr, 0xaa, SRAM_SIZE); // set All to AA's
		for(i=0, ptr=sram_base_ptr; i<SRAM_SIZE; i++, ptr++) {
			if(*ptr != 0xaa) return(-1);
		}
		memset((void*)sram_base_ptr, 0x55, SRAM_SIZE); // set All to 55's
		for(i=0, ptr=sram_base_ptr; i<SRAM_SIZE; i++, ptr++) {
			if(*ptr != 0x55) return(-1);
		}
		for(i=0, iptr=(int*)sram_base_ptr; i<SRAM_SIZE/sizeof(int); i++, iptr++) { // Seq number test.
			*iptr = i;
		}	
	} else {
		printf("Reading SRAM\n");
		for(i=0, iptr=(int*)sram_base_ptr; i<SRAM_SIZE/sizeof(int); i++, iptr++) { // Seq number test.
			if(*iptr != i)  {
				 printf("value should be: %d is %d @addr: %d\n", i,*iptr, iptr - (int*)sram_base_ptr);
				return(-1);
			}
			if (read == 2) {	// Verbose.
				 printf("value should be: %d is %d @addr: %d\n", i,*iptr, iptr - (int*)sram_base_ptr);
			}
		}
	}
	return(0);
}

unsigned char* sram_open(void) {
	int mem_fd;
	unsigned char *sram_mem;
	
	/* open character device(1,1) '/dev/mem' */
    	if ((mem_fd = open("/dev/mem", O_RDWR) ) < 0) {
        	printf("sram_open: can't open /dev/mem \n");
	        exit (-1);
	}

	/* malloc space in our memory */
	if ((sram_mem = malloc(SRAM_SIZE + (PAGE_SIZE-1))) == NULL) {
        	printf("sram_open: malloc allocation error \n");
	        	exit (-1);
	}
	if ((unsigned long)sram_mem % PAGE_SIZE) {
		sram_mem += PAGE_SIZE - ((unsigned long)sram_mem % PAGE_SIZE);
	}
    
	/* mmap static ram into our memory space */
	sram_mem = (unsigned char *)mmap(
        	(caddr_t)sram_mem, 
	            SRAM_SIZE,
	            PROT_READ|PROT_WRITE,
	            MAP_SHARED|MAP_FIXED,
	            mem_fd, 
	            SRAM_BASE
	);
	if ((long)sram_mem < 0) {
		printf("sram_open: mmap error \n");
		exit (-1);
	}
	return sram_mem;
}


